about summary refs log tree commit diff
path: root/sysdeps/s390/s390-64/memcpy.S
diff options
context:
space:
mode:
Diffstat (limited to 'sysdeps/s390/s390-64/memcpy.S')
-rw-r--r--sysdeps/s390/s390-64/memcpy.S34
1 files changed, 19 insertions, 15 deletions
diff --git a/sysdeps/s390/s390-64/memcpy.S b/sysdeps/s390/s390-64/memcpy.S
index c59d3fbb79..008f026909 100644
--- a/sysdeps/s390/s390-64/memcpy.S
+++ b/sysdeps/s390/s390-64/memcpy.S
@@ -1,7 +1,6 @@
-/* Set a block of memory to some byte value.  64 bit S/390 version.
-   Copyright (C) 2001 Free Software Foundation, Inc.
+/* memcpy - copy a block from source to destination.  64 bit S/390 version.
+   Copyright (C) 2000, 2001 Free Software Foundation, Inc.
    Contributed by Martin Schwidefsky (schwidefsky@de.ibm.com).
-   This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public License as
@@ -19,8 +18,8 @@
    Boston, MA 02111-1307, USA.  */
 
 /* INPUT PARAMETERS
-     %r2 = address to destination memory area
-     %r3 = address to source memory area
+     %r2 = address of destination memory area
+     %r3 = address of source memory area
      %r4 = number of bytes to copy.  */
 
 #include "sysdep.h"
@@ -28,14 +27,19 @@
 
 	.text
 ENTRY(memcpy)
-	ltgr	%r5,%r4
+	ltgr	%r4,%r4
+	jz	.L3
+	aghi	%r4,-1		    # length - 1
+	lgr	%r1,%r2		    # copy destination address
+	srag	%r5,%r4,8
 	jz	.L1
-	lgr	%r4,%r3		    # %r4/%r5 = source ptr/len
-	lgr	%r3,%r5		    # %r2/%r3 = dest ptr/len
-	lgr	%r0,%r2		    # save source address
-.L0:	mvcle	%r2,%r4,0	    # thats it, MVCLE is your friend
-	jo	.L0
-	lgr	%r2,%r0		    # return value is source address
-.L1:
-	br	%r14
-END(memset)
+.L0:	mvc	0(256,%r1),0(%r3)   # move in 256 byte chunks
+	la	%r1,256(%r1)
+	la	%r3,256(%r3)
+	brctg	%r5,.L0
+.L1:	bras	%r5,.L2		    # setup base pointer for execute
+	mvc	0(1,%r1),0(%r3)	    # instruction for execute
+.L2:	ex	%r4,0(%r5)	    # execute mvc with length ((%r4)&255)+1
+.L3:	br	%r14
+END(memcpy)
+